// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
var app = angular.module('hd', ['ionic', 'ngCordova', 'hd.controllers', 'hd.directives', 'hd.tickets', 'hd.settings', 'hd.comments', 'ngSanitize'])

app.run(function($ionicPlatform, $cordovaStatusbar, $rootScope, $ionicLoading, Settings) {
    if (Settings.valid()) {
        Settings.initHttpWithCurrentSettings();
    }

    $ionicPlatform.ready(function() {

        //init settings
        if (Settings.valid()) {
            Settings.loadUser();
        }
    });

    $rootScope.$on('loading:show', function() {
        $ionicLoading.show();
    })

    $rootScope.$on('loading:hide', function() {
        $ionicLoading.hide()
        $rootScope.$broadcast('scroll.infiniteScrollComplete');
        $rootScope.$broadcast('scroll.refreshComplete');
    })
})

app.config(function($stateProvider, $urlRouterProvider) {
    $urlRouterProvider.otherwise('/tickets')

    $stateProvider.state('tickets', {
        url: '/tickets',
        templateUrl: 'templates/ticket-list.html',
        controller: "TicketsCtrl"
    });

    $stateProvider.state('tickets-details', {
        url: '/ticket/:ticketId',
        templateUrl: 'templates/ticket-details.html',
        controller: "TicketDetailCtrl"
    });

    $stateProvider.state('ticket-meta', {
        url: '/ticket/:ticketId/meta',
        templateUrl: 'templates/ticket-meta.html',
        controller: "TicketMetaCtrl"
    });

    $stateProvider.state('new-ticket', {
        url: '/tickets/new',
        templateUrl: 'templates/ticket-new.html',
        controller: "NewTicketCtrl"
    });

    $stateProvider.state('settings', {
        url: '/settings',
        templateUrl: 'templates/settings.html',
        controller: "SettingsCtrl"
    });
});

app.config(function($httpProvider) {
    $httpProvider.interceptors.push(function($rootScope, $q, $location) {
        return {
            request: function(config) {
                $rootScope.$broadcast('loading:show');
                return config
            },
            response: function(response) {
                $rootScope.$broadcast('loading:hide');
                if(typeof response.data == 'string' && response.data.indexOf("<html")>-1)
                    $location.path("/settings")
                return response
            },
            requestError: function(rejection) {
                $rootScope.$broadcast('loading:hide');
                return $q.reject(rejection);
            },
            responseError: function(rejection) {
                // do something on error
                $rootScope.$broadcast('loading:hide');
                if(rejection.status == 403)
                    $location.path("/settings");
                else
                    alert(error.statusText);       
                return $q.reject(rejection);
            }
        }
    })
});
