app.controller('TicketsCtrl', function($scope, $rootScope, $state, $ionicPopup, Tickets, Settings) {
    $scope.viewMode = localStorage.getItem("viewMode") || "unanswered";
    $scope.tickets = [];
    $scope.showMore = false;

    $scope.updateViewMode = function(mode) {
        viewModePopup.close();
        $scope.viewMode = mode;
        localStorage.setItem("viewMode", mode);
        $scope.tickets = [];
        $scope.loadTickets();
    }

    $scope.refreshTickets = function() {
        $scope.showMore = false;
        $scope.tickets = [];
        $scope.loadTickets();
    }
    $scope.loadTickets = function(refresh) {

        Tickets.loadTickets({
            'mode': $scope.viewMode,
            'offset': $scope.tickets.length
        }, function(tickets) {
            Array.prototype.push.apply($scope.tickets, tickets);

            if (tickets.length == 20)
                $scope.showMore = true;
            else
                $scope.showMore = false;
        });
    }

    var viewModePopup = null;
    $scope.showPopup = function() {
        $scope.data = {}

        // An elaborate, custom popup
        viewModePopup = $ionicPopup.show({
            templateUrl: 'templates/viewmode-selector.html',
            title: 'Choose the view mode',
            scope: $scope,
            buttons: [{
                text: 'Cancel'
            }]
        });
    };

    $scope.isOverdue = function(date) {
        return moment(date).diff(moment()) < 0;
    }

    $scope.delete = function(id) {
        var confirmPopup = $ionicPopup.confirm({
            title: 'Delete ticket',
            template: 'Are you sure you want to delete this ticket?'
        });
        confirmPopup.then(function(res) {
            if (res) {
                Tickets.deleteTicket(id).then(
                    function() {
                        $scope.refreshTickets();
                    });
            }
        });
    }

    $scope.spam = function(id) {
        var confirmPopup = $ionicPopup.confirm({
            title: 'Delete ticket',
            template: 'Are you sure you want to mark this ticket as spam?'
        });
        confirmPopup.then(function(res) {
            if (res) {
                Tickets.spamTicket(id).then(
                    function() {
                        $scope.refreshTickets();
                    });
            }
        });
    }

    $scope.close = function(id) {
        var confirmPopup = $ionicPopup.confirm({
            title: 'Close ticket',
            template: 'Are you sure you want to close this ticket?'
        });
        confirmPopup.then(function(res) {
            if (res) {
                Tickets.setStatus(id, 3).then(
                    function() {
                        $scope.refreshTickets();
                    });
            }
        });
    }

    $rootScope.$on('settingsChanged', function(event, args) { $scope.refreshTickets() }); //refresh when the settings are changed

    if (!Settings.valid()) {
        $state.go("settings");
        return;
    }

    //IMPORTANT: This call should always be the last one in this controller
    $scope.loadTickets();
});
