angular.module('hd.settings', [])
    .service('Settings', function($http, $cordovaPush, $q, $ionicPlatform) {
        var getSettings = function() {
            var settings = {}
            settings.username = localStorage.getItem("hduname");
            settings.password = localStorage.getItem("hdpsw");
            settings.url = localStorage.getItem("hdurl");
            if (!settings.url) settings.url = "http://"
            return settings;
        };

        var settings = getSettings();
        var user = {};

        var makeBasicAuthHeader = function(username, password) {
            return "Basic " + btoa(username + ":" + password);
        }

        var getBasicAuthHeader = function() {
            return makeBasicAuthHeader(settings.username, settings.password);
        }

        var initHttpWithCurrentSettings = function() {
            if (valid()) {
                $http.defaults.headers.common.Authorization = getBasicAuthHeader();
            }
        }

        var valid = function() {
            return !!(settings.username && settings.password && settings.url);
        }

        var getRequestUrl = function(path) {
            //TODO: make a proper URL combine
            return (settings.url + path);
        }

        var cannedResponses = null;
        var getCannedResponses = function() {
            if (cannedResponses !== null) return cannedResponses;
            $http.get(getRequestUrl("/api/CommentTemplates")).then(function(resp) {
                cannedResponses = resp.data;
                return cannedResponses;
            });
        }

        var loadUser = function() {
            var deferred = $q.defer();
            $http.post(getRequestUrl('api/Authorization')).
            success(function(data, status, headers, config) {
                user = data;
                registerPush();
                getCannedResponses();
                deferred.resolve();
                initHttpWithCurrentSettings();
            }).
            catch(function(data, status, headers, config) {
                deferred.reject(data);
            });

            return deferred.promise;
        }

        var registerPush = function() {
            if (!(window.plugins && window.plugins.pushNotification))
                return;
            if (!$ionicPlatform.is("ios"))
                return; //ios only for now

            var iosConfig = {
                "badge": "false",
                "sound": "true",
                "alert": "true",
            };

            $cordovaPush.register(iosConfig).then(function(result) {
                $http.post(getRequestUrl("/api/SavePushToken"), {
                    token: result
                });
            }, function(err) {
                alert("Error initiating push notifications: " + err);
            });
        }


        var getUser = function() {
            return user;
        }

        var saveSettings = function(localSettings) {
            var deferred = $q.defer();
            //normalize url
            if (localSettings.url.indexOf("/", localSettings.url.length - 1) == -1)
                localSettings.url += "/";
            if (localSettings.url.indexOf("http") == -1) {
                localSettings.url = "http://" + localSettings.url;
            }
            if (localSettings.url.indexOf(".jitbit.com") > -1) {
                if (localSettings.url.indexOf("/helpdesk") == -1) {
                    localSettings.url = localSettings.url + "helpdesk/";
                }
            }

            settings = localSettings;
            localStorage.setItem("hduname", localSettings.username);
            localStorage.setItem("hdpsw", localSettings.password);
            localStorage.setItem("hdurl", localSettings.url);
            initHttpWithCurrentSettings();
            loadUser().then(function() {
                deferred.resolve();
            }, function(error) {
                deferred.reject(error);
            });

            return deferred.promise;
        }
        return {
            saveSettings: saveSettings,
            getSettings: getSettings,
            getBasicAuthHeader: getBasicAuthHeader,
            getRequestUrl: getRequestUrl,
            valid: valid,
            initHttpWithCurrentSettings: initHttpWithCurrentSettings,
            loadUser: loadUser,
            getUser: getUser,
            getCannedResponses: getCannedResponses
        }
    })
