angular.module('hd.tickets', [])
    .service('Tickets', function($http, $q, $cordovaFileTransfer, Settings) {
        var tickets = [];
        var currentTicket = null;
        var loadTickets = function(params, callback) {
            params.count = 20;
            $http.get(Settings.getRequestUrl("/api/tickets"), {
                params: params
            }).then(function(resp) {
                    if (params.offset)
                        Array.prototype.push.apply(tickets, resp.data); //more button pressed add tickets to cache
                    else
                        tickets = resp.data;

                    callback(resp.data);
                },
                function(err) {});
        };

        var getStatuses = function(ticketId, callback) {
            $http.get(Settings.getRequestUrl("api/GetAvailableStatuses"), {
                params: {
                    'id': ticketId
                }
            }).success(function(data) {
                callback(data);
            });
        }

        var setStatus = function(ticketId, statusId) {
            var deferred = $q.defer();

            $http.post(Settings.getRequestUrl("/api/SetTicketStatus"), {
                    id: ticketId,
                    statusId: statusId
                })
                .success(function(data) {
                    deferred.resolve();
                }).
            error(function(data, status, headers, config) {
                deferred.reject(data);
            });

            return deferred.promise;
        }

        var deleteTicket = function(ticketId) {
            var deferred = $q.defer();

            $http.post(Settings.getRequestUrl("api/DeleteTicket"), {
                    id: ticketId
                })
                .success(function(data) {
                    deferred.resolve();
                }).
            error(function(data, status, headers, config) {
                deferred.reject(data);
            });

            return deferred.promise;
        }

        var spamTicket = function(ticketId) {
            var deferred = $q.defer();

            $http.post(Settings.getRequestUrl("api/MarkSpam"), {
                    id: ticketId
                })
                .success(function(data) {
                    deferred.resolve();
                }).
            error(function(data, status, headers, config) {
                deferred.reject(data);
            });

            return deferred.promise;
        }

        var uploadFile = function(imageURI, mimeType, ticketId) {
            try {
                var options = new FileUploadOptions();
                options.fileKey = "file";
                //get filename and get file extenstion from mimetype (android does not give you file ext)
                options.fileName = imageURI.substr(imageURI.lastIndexOf('/') + 1);
                if (mimeType)
                    options.fileName += "." + mimeType.substr(mimeType.lastIndexOf('/') + 1);

                options.mimeType = mimeType;
                options.headers = {
                    "Authorization": Settings.getBasicAuthHeader()
                };

                var params = new Object();

                options.params = params;
                $cordovaFileTransfer.upload(Settings.getRequestUrl("api/AttachFile?id=" + ticketId), imageURI, options).then(
                    function(r) {
                        alert("Upload successful");
                    },
                    function(error) {
                        alert("An error has occurred: Code = " + error.code);
                    });
            } catch (err) {
                alert(err.message);
            }
        }

        return {
            all: tickets,
            loadTickets: loadTickets,
            getTicket: function(ticketId, callback) {
                var ticket = {};
                $http.get(Settings.getRequestUrl("/api/ticket/" + ticketId)).then(function(resp) {
                        ticket = resp.data;
                        callback(resp.data);
                    },
                    function(err) {});
            },
            createTicket: function(ticket, callback) {
                var deferred = $q.defer();
                $http.post(Settings.getRequestUrl("/api/ticket"), {
                    subject: ticket.subject,
                    body: ticket.body,
                    priorityId: 0,
                    categoryId: ticket.categoryId,
                    customFields: ticket.customFields
                }).success(function() {
                    deferred.resolve();
                }).error(function() {
                    deferred.reject();
                });;

                return deferred.promise;
            },
            getCategories: function(callback) {
                $http.get(Settings.getRequestUrl("/api/categories")).then(function(resp) {
                    callback(resp.data);
                })
            },
            getStatuses: getStatuses,
            setStatus: setStatus,
            deleteTicket: deleteTicket,
            spamTicket: spamTicket,
            getTicketFromCache: function(ticketId) {
                for (var i = tickets.length - 1; i >= 0; i--) {
                    if (tickets[i].IssueID == ticketId)
                        return tickets[i];
                };
            },
            uploadFile: uploadFile,
            currentTicket: function() {
                return currentTicket;
            },
            setSetCurrentTicket: function(value) {
                currentTicket = value;
            },
            getCustomFields: function(ticketId, callback) {
                var ticket = {};
                $http.get(Settings.getRequestUrl("/api/ticketCustomFields/" + ticketId)).then(function(resp) {
                        ticket = resp.data;
                        callback(resp.data);
                    },
                    function(err) {});
            },
            setPriority: function(ticketId, priority) {
                var deferred = $q.defer();
                $http.post(Settings.getRequestUrl("/api/setPriority"), {
                    id: ticketId,
                    priority: priority
                }).success(function() {
                    deferred.resolve();
                }).error(function() {
                    deferred.reject();
                });;

                return deferred.promise;
            },
            setCategory: function(ticketId, categoryId) {
                var deferred = $q.defer();
                $http.post(Settings.getRequestUrl("/api/setCategory"), {
                    id: ticketId,
                    categoryId: categoryId
                }).success(function() {
                    deferred.resolve();
                }).error(function() {
                    deferred.reject();
                });;

                return deferred.promise;
            },
            getTechsForCategory: function(categoryId, callback) {
                $http.get(Settings.getRequestUrl("/api/techsForCategory/" + categoryId)).then(function(resp) {
                        callback(resp.data);
                    },
                    function(err) {});
            },
            setAssignee: function(ticketId, userId) {
                var deferred = $q.defer();
                $http.post(Settings.getRequestUrl("/api/setAssignee"), {
                    id: ticketId,
                    userId: userId
                }).success(function() {
                    deferred.resolve();
                }).error(function() {
                    deferred.reject();
                });;

                return deferred.promise;
            },

            getCustomFieldsForCategory: function(categoryId, callback) {
                $http.get(Settings.getRequestUrl("/api/CustomFieldsForCategory/?categoryId=" + categoryId)).then(function(resp) {
                        callback(resp.data);
                    },
                    function(err) {});
            }

        }
    })
